#include "boardview.h"
#include <vbase/array.h>

/* copyright by Tim vor der Brck, distributed under the GNU General Public License */
//************************* Boardview **********************
CBoardview::CBoardview()
{
  m_pmemptyboard=0;
  m_pmboard=0;
  m_pmdnd=0;
  m_dx=0;
  m_dy=0;
  m_columns=0;
  m_rows=0;
  m_bonscreen=true;
  m_win=0;
}

//*********************** ~Boardview ***********************
CBoardview::~CBoardview()
{
  if (m_pmemptyboard!=0) XFreePixmap(m_display,m_pmemptyboard);
  if (m_pmboard!=0) XFreePixmap(m_display,m_pmboard);
  if (m_pmdnd!=0) XFreePixmap(m_display,m_pmdnd);
  if (m_gc!=0) XFreeGC(m_display,m_gc);
}

//*********************** create ****************************
/**
 * This initializes a boardview object
 *
 * @param g  game class that implements the methods drawboard and convvaluetoicon
 * @param board  logical board, that must implements the method getcontents
 * @param display  graphical display
 * @param win  window to draw to
 * @param width  width of the window
 * @param height  height of the window 
 * @param columns  number of columns of the board+2
 * @param rows  number of rows of the board+2 (need a little space outside the board)
 */
void CBoardview::create(CGame* g,CBoard* board,Display* display,Window win,
                      int width,int height,int columns,int rows)
{
  // init variables
  m_game=g;
  m_display=display;
  m_screennr=DefaultScreen(m_display);
  m_defroot=DefaultRootWindow(m_display);
  m_root=RootWindow(m_display,m_screennr);
  m_depth=DefaultDepth(m_display,m_screennr);
  m_win=win;
  m_board=board;
  m_dx=width;
  m_dy=height;
   
 
  // sets m_columns,m_rows,allocates m_contents, set fieldwidth,height
  setdimensions(columns,rows);
  
  // create GC
  m_gc=XCreateGC(m_display,m_root,0,NULL);
  
  // delete all entrys of m_contents
  deleteboard();

  // allocate pixmaps
  setuppixmaps(); 

  // draws empty board
  drawemptyboard();

}

//***************************** resize *************************
/* 
 * resizes the window
 * calls draw automatically afterwards
 * @param width  new width
 * @param height  new height
 */
void CBoardview::resize(int width,int height)
{
  m_dx=width;
  m_dy=height;
 
  // init field width,height
  if (m_columns!=0)
  {
    m_fdx=m_dx/m_columns;
    m_fdy=m_dy/m_rows;
  }
  
  setuppixmaps();  
  redraw();
}

/* changes dimensions */
void CBoardview::changedimensions(int columns,int rows)
{
  setdimensions(columns,rows);
  redraw();
}

void CBoardview::setuppixmaps()
{
  if (m_pmemptyboard!=0) XFreePixmap(m_display,m_pmemptyboard);
  if (m_pmboard!=0) XFreePixmap(m_display,m_pmboard);
  if (m_pmdnd!=0) XFreePixmap(m_display,m_pmdnd);
  m_pmemptyboard=XCreatePixmap(m_display,m_defroot,m_dx,m_dy,m_depth);
  m_pmboard=XCreatePixmap(m_display,m_defroot,m_dx,m_dy,m_depth);
  m_pmdnd=XCreatePixmap(m_display,m_defroot,m_dx,m_dy,m_depth);
}

/**
 * draws an empty board with no pieces on it
 */
void CBoardview::drawemptyboard()
{
  m_game->drawboard(m_display,m_gc,m_pmemptyboard,m_dx,m_dy);
  XCopyArea(m_display,m_pmemptyboard,m_pmboard,m_gc,0,0,m_dx,m_dy,0,0);
  XCopyArea(m_display,m_pmemptyboard,m_pmdnd,m_gc,0,0,m_dx,m_dy,0,0);
}
  
/**
 * redraws the board
 * in first drawing an empty board and then the pieces on it,
 * it uses double buffering
 * for instance you need to call this function if you resize the board
 */
void CBoardview::redraw() // the thorough way
{
  m_bonscreen=false;
  int x,y;
  CCounter* ss;
  drawemptyboard();
  for (x=0;x<m_columns;x++)
  {
    for (y=0;y<m_rows;y++)
    {
      ss=m_contents[x][y];
      if (ss!=NULL) ss->draw();
    }
  }
  m_bonscreen=true;
  draw();  
}

/* sets how many couloms and rows the board has, you should call redraw afterwards */ 
void CBoardview::setdimensions(int columns,int rows)
{
  if (m_columns!=0) FreeMatrix(m_contents,m_columns);
  m_rows=rows;
  m_columns=columns;
  if (m_columns!=0 && m_rows!=0)
  {
    m_fdx=m_dx/m_columns;
    m_fdy=m_dy/m_rows;
  }
  AllocMatrix(m_contents,columns,rows);
}
  
/* sets the output window, you should call redraw afterwards */
void CBoardview::setwindow(Window win)
{
  m_win=win;
}  

void CBoardview::removeicon(int x,int y,CIcon* icon)
{
  int winx=convxtowinx(x,true,icon->getwidth());
  int winy=convytowiny(y,true,icon->getheight());
  icon->remove(winx,winy,m_pmemptyboard,m_pmboard);
  icon->remove(winx,winy,m_pmemptyboard,m_pmdnd);
  if (m_bonscreen) icon->remove(winx,winy,m_pmemptyboard,m_win);
}

/* draws an icon at the specified position */
void CBoardview::drawicon(int x,int y,CIcon* icon)
{
  int winx=convxtowinx(x,true,icon->getwidth());
  int winy=convytowiny(y,true,icon->getheight());
  icon->draw(winx,winy,m_pmboard);
  icon->draw(winx,winy,m_pmdnd);
  if (m_bonscreen) icon->draw(winx,winy,m_win);
}
  
/* returns the contents of field x,y */
CCounter* CBoardview::getcontents(int x,int y)
{
  return m_contents[x][y];
}
   
/* set contents of field x,y with counter ss */
void CBoardview::setcontents(int x,int y,CCounter* ss)
{
  m_contents[x][y]=ss;
} 
  

/* draws a part of the board */ 
void CBoardview::draw(int x,int y,int width,int height)
{
  if (m_win!=0) XCopyArea(m_display,m_pmboard,m_win,m_gc,x,y,width,height,x,y);
}

/* converts a board to a window position */
int CBoardview::convxtowinx(int x,bool centre,int width)
{
	int winx=x*m_fdx;
	if (centre)
	{
		winx=winx+(m_fdx-width)/2;
	}
	return winx;

}

int CBoardview::convytowiny(int y,bool centre,int height)
{
  int winy=m_dy-m_fdy*y;
  if (centre)
  {
    winy=winy-height-(m_fdy-height)/2;
   }
   return winy;
}

int CBoardview::convwinxtox(int winx,bool centre,int width)
{
  int x;
  if (centre)
  {
     winx=winx-(m_fdx-width)/2;
   }
   x=winx/m_fdx;
   return x;
}

int CBoardview::convwinytoy(int winy,bool centre,int height)
{
  int y;
  if (centre)
  {
     winy=winy+height-(m_fdy-height)/2;
   }
   winy=m_dy-winy;
   y=winy/m_fdy;
   return y;
}


//*************************** begindrag *************************
/* winx,winy: position of the mousepointer
   x,y: position of the stone on the board 1: for the first field on the actual board*/
bool CBoardview::begindrag(int winx,int winy,int x,int y,bool bmove)
{
  CCounter* ss=m_contents[x][y];
  if (ss==NULL) return false;
  CIcon* picon=ss->geticon();
  int iconx=convxtowinx(x,true,picon->getwidth());
  int icony=convytowiny(y,true,picon->getheight());

  //delete source icon by move operation
  if (bmove)
  {
     picon->remove(iconx,icony,m_pmemptyboard,m_pmboard);
     picon->remove(iconx,icony,m_pmemptyboard,m_pmboard);
  }

  m_hotx=iconx-winx;
  m_hoty=icony-winy;
  picon->begindrag(iconx,icony,m_pmboard,m_pmdnd);
  picon->dbdraw(m_pmdnd,m_win);  
  m_dragicon=picon;
  return true;
}

void CBoardview::dodrag(int winx,int winy)
{
  m_dragicon->dodrag(winx+m_hotx,winy+m_hoty);
  m_dragicon->dbdraw(m_pmdnd,m_win);
}

void CBoardview::drop(int winx,int winy,int& x,int& y)
{
  m_dragicon->drop();
  m_dragicon->dbdraw(m_pmdnd,m_win);
  x=convwinxtox(winx,false,0);
  y=convwinytoy(winy,false,0);
}

/* updates the look of the board, used by creation
   of the board,by loading a game and eventually by moving
   a piece*/
void CBoardview::update()
{

  int x,y;
  int valueview; //value of a counter in the view
  int valueboard; //value of a counter in the board
  CCounter* ss; //Counter in the view

  m_bonscreen=false;
  
  // for all fields inside the board 
  for (x=1;x<=m_columns-2;x++)
  {
    for (y=1;y<=m_rows-2;y++)
    {
       // get value of counter viewing on the window
       ss=getcontents(x,y);
       if (ss!=NULL) valueview=ss->getvalue();
         else valueview=0;

       // get value of counter of logical board
       valueboard=m_board->getcontents(x-1,y-1);

       // contents of board different from view?
       if (valueboard!=valueview)
       {
          if (valueboard==0) delete ss; 
	  else m_contents[x][y]=new CCounter(x,y,m_game->convvaluetoicon(valueboard),valueboard,this);
       }
       else if (valueview!=0) ss->draw();
     }
  }

  //und zeichnen
  m_bonscreen=true;
}

/* sets all fields to NULL */
void CBoardview::deleteboard()
{
int x,y;
for (x=0;x<m_columns;x++)
  for (y=0;y<m_rows;y++)
    m_contents[x][y]=NULL;

}

/* sets all fields to NULL and erases any NOT NULL entrys */
void CBoardview::eraseboard()
{
  CCounter* ss;
  m_bonscreen=false;
  for (int x=1;x<m_columns-1;x++)
  {
    for (int y=1;y<m_rows-1;y++)
    {
      ss=m_contents[x][y];
      if (ss!=NULL) delete(ss);
    }
  }
  m_bonscreen=true;
}

void CBoardview::draw()
{
  if (m_win!=0) XCopyArea(m_display,m_pmboard,m_win,m_gc,0,0,m_dx,m_dy,0,0);
}
 







